@@ -33,9 +33,16 @@
 #include "vino-input.h"
 #include "vino-cursor.h"
 #include "vino-prompt.h"
+#include "vino-status-icon.h"
+#include "vino-dbus-listener.h"
 #include "vino-util.h"
 #include "vino-enums.h"
 #include <sys/poll.h>
+#include <dbus/dbus-glib.h>
+
+#ifdef VINO_ENABLE_KEYRING
+#include <gnome-keyring.h>
+#endif
 
 /* If an authentication attempt failes, delay the next
  * authentication attempt for 5 seconds.
@@ -50,6 +57,8 @@ struct _VinoServerPrivate
   VinoFB           *fb;
   VinoCursorData   *cursor_data;
   VinoPrompt       *prompt;
+  VinoStatusIcon   *icon;
+  VinoDBusListener *listener;
 
   GIOChannel       *io_channel;
   guint             io_watch;
@@ -63,14 +72,19 @@ struct _VinoServerPrivate
   VinoHTTP         *http;
 #endif
 
+  int               alternative_port;
+
   guint             on_hold : 1;
+  guint             local_only : 1;
   guint             prompt_enabled : 1;
   guint             view_only : 1;
   guint             require_encryption : 1;
   guint             last_auth_failed : 1;
+  guint             use_alternative_port : 1;
+  guint             lock_screen : 1;
 };
 
-typedef struct
+struct _VinoClient
 {
   rfbClientPtr  rfb_client;
   GIOChannel   *io_channel;
@@ -81,7 +95,7 @@ typedef struct
   guint         auth_timeout;
   char         *auth_response;
   int           auth_resp_len;
-} VinoServerClientInfo;
+};
 
 enum
 {
@@ -90,27 +104,97 @@ enum
   PROP_ON_HOLD,
   PROP_PROMPT_ENABLED,
   PROP_VIEW_ONLY,
+  PROP_LOCAL_ONLY,
+  PROP_USE_ALTERNATIVE_PORT,
+  PROP_ALTERNATIVE_PORT,
   PROP_REQUIRE_ENCRYPTION,
   PROP_AUTH_METHODS,
-  PROP_VNC_PASSWORD
+  PROP_VNC_PASSWORD,
+  PROP_PORT,
+  PROP_LOCK_SCREEN
 };
 
-static enum rfbNewClientAction vino_server_auth_client (VinoServer           *server,
-							VinoServerClientInfo *client,
-							const char           *response,
-							int                   length);
+static enum rfbNewClientAction vino_server_auth_client (VinoServer *server,
+							VinoClient *client,
+							const char *response,
+							int         length);
 
-static void vino_server_setup_framebuffer   (VinoServer *server);
-static void vino_server_release_framebuffer (VinoServer *server);
+static void vino_server_setup_framebuffer     (VinoServer *server);
+static void vino_server_release_framebuffer   (VinoServer *server);
+static void vino_server_update_security_types (VinoServer *server);
 
 static gpointer parent_class;
 
+static void
+vino_server_lock_screen (VinoServer *server)
+{
+#define GNOME_SCREENSAVER_BUS_NAME  "org.gnome.ScreenSaver"
+#define GNOME_SCREENSAVER_INTERFACE "org.gnome.ScreenSaver"
+#define GNOME_SCREENSAVER_PATH      "/org/gnome/ScreenSaver"
+
+  DBusGConnection *connection;
+  GError          *error;
+  DBusGProxy      *proxy;
+
+  if (!server->priv->lock_screen)
+    return;
+  
+  dprintf(DBUS, "Locking screen via gnome-screensaver\n");
+
+  error = NULL;
+  connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+  if (!connection)
+    {
+      g_printerr (_("Failed to open connection to bus: %s\n"),
+                  error->message);
+      g_error_free (error);
+      return;
+    }
+
+  proxy = dbus_g_proxy_new_for_name (connection,
+                                     GNOME_SCREENSAVER_BUS_NAME,
+                                     GNOME_SCREENSAVER_PATH,
+                                     GNOME_SCREENSAVER_INTERFACE);
+
+  dbus_g_proxy_call_no_reply (proxy, "Lock", G_TYPE_INVALID);
+
+  g_object_unref (proxy);
+  dbus_g_connection_unref (connection);
+
+#undef GNOME_SCREENSAVER_BUS_NAME
+#undef GNOME_SCREENSAVER_INTERFACE
+#undef GNOME_SCREENSAVER_PATH
+}
+
+
+static void
+vino_server_client_accepted (VinoServer *server,
+                             VinoClient *client)
+{
+  if (!server->priv->icon)
+    server->priv->icon = vino_status_icon_new (server, server->priv->screen);
+
+  vino_status_icon_add_client (server->priv->icon, client);
+}
+
+static void
+vino_server_client_disconnected (VinoServer *server,
+                                 VinoClient *client)
+{
+  if (server->priv->icon)
+    if (vino_status_icon_remove_client (server->priv->icon, client))
+      {
+        g_object_unref (server->priv->icon);
+        server->priv->icon = NULL;
+
+        vino_server_lock_screen (server);
+      }
+}
 static void
 vino_server_handle_client_gone (rfbClientPtr rfb_client)
 {
-  VinoServer           *server = VINO_SERVER (rfb_client->screen->screenData);
-  VinoServerClientInfo *client = NULL;
-  GSList               *l;
+  VinoServer *server = VINO_SERVER (rfb_client->screen->screenData);
+  GSList     *l;
 
   g_return_if_fail (VINO_IS_SERVER (server));
 
@@ -120,7 +204,7 @@ vino_server_handle_client_gone (rfbClientPtr rfb_client)
 
   for (l = server->priv->clients; l; l = l->next)
     {
-      client = (VinoServerClientInfo *) l->data;
+      VinoClient *client = (VinoClient *) l->data;
 
       if (rfb_client == client->rfb_client)
 	{
@@ -139,7 +223,8 @@ vino_server_handle_client_gone (rfbClientPtr rfb_client)
 	    g_source_remove (client->update_timeout);
 	  client->update_timeout = 0;
 
-	  g_source_remove (client->io_watch);
+          if (client->io_watch)
+            g_source_remove (client->io_watch);
 	  client->io_watch = 0;
 
 	  g_io_channel_unref (client->io_channel);
@@ -147,6 +232,9 @@ vino_server_handle_client_gone (rfbClientPtr rfb_client)
 
 	  g_free (l->data);
 	  server->priv->clients = g_slist_delete_link (server->priv->clients, l);
+
+          vino_server_client_disconnected (server, client);
+
 	  break;
 	}
     }
@@ -201,39 +289,6 @@ vino_server_update_client_timeout (rfbClientPtr rfb_client)
   return TRUE;
 }
 
-static void
-vino_server_set_client_on_hold (VinoServer            *server,
-				VinoServerClientInfo  *client,
-				gboolean               on_hold)
-{
-  rfbClientPtr rfb_client = client->rfb_client;
-
-  dprintf (RFB, "Setting client '%s' on hold: %s\n",
-	   rfb_client->host, on_hold ? "(true)" : "(false)");
-
-  rfb_client->onHold = on_hold;
-
-  /* We don't process any pending data from an client which is
-   * on hold, so don't let it starve the rest of the mainloop.
-   */
-  g_source_set_priority (g_main_context_find_source_by_id (NULL, client->io_watch),
-			 on_hold ? G_PRIORITY_LOW : G_PRIORITY_DEFAULT);
-
-  if (!on_hold)
-    {
-      if (!client->update_timeout)
-	client->update_timeout = g_timeout_add (50,
-						(GSourceFunc) vino_server_update_client_timeout,
-						rfb_client);
-    }
-  else
-    {
-      if (client->update_timeout)
-	g_source_remove (client->update_timeout);
-      client->update_timeout = 0;
-    }
-}
-
 static inline gboolean
 more_data_pending (int fd)
 {
@@ -257,11 +312,56 @@ vino_server_client_data_pending (GIOChannel   *source,
   return vino_server_update_client (rfb_client);
 }
 
+static void
+vino_server_set_client_on_hold (VinoServer  *server,
+				VinoClient  *client,
+				gboolean     on_hold)
+{
+  rfbClientPtr rfb_client = client->rfb_client;
+
+  dprintf (RFB, "Setting client '%s' on hold: %s\n",
+	   rfb_client->host, on_hold ? "(true)" : "(false)");
+
+  rfb_client->onHold = on_hold;
+
+  if (on_hold)
+    {
+      if (client->io_watch)
+        {
+          g_source_remove (client->io_watch);
+          client->io_watch = 0;
+        }
+
+      if (client->update_timeout)
+        {
+          g_source_remove (client->update_timeout);
+          client->update_timeout = 0;
+        }
+    }
+  else
+    {
+      if (!client->io_watch)
+        {
+          client->io_watch = g_io_add_watch (client->io_channel,
+					     G_IO_IN|G_IO_PRI,
+					     (GIOFunc) vino_server_client_data_pending,
+					     rfb_client);
+        }
+
+      if (!client->update_timeout)
+        {
+          client->update_timeout = g_timeout_add (50,
+                                                  (GSourceFunc) vino_server_update_client_timeout,
+                                                  rfb_client);
+        }
+    }
+}
+
 static enum rfbNewClientAction
 vino_server_handle_new_client (rfbClientPtr rfb_client)
 {
-  VinoServer           *server = VINO_SERVER (rfb_client->screen->screenData);
-  VinoServerClientInfo *client;
+  VinoServer *server = VINO_SERVER (rfb_client->screen->screenData);
+  VinoClient *client;
 
   g_return_val_if_fail (VINO_IS_SERVER (server), RFB_CLIENT_REFUSE);
 
@@ -270,7 +370,7 @@ vino_server_handle_new_client (rfbClientPtr rfb_client)
   if (!server->priv->fb)
     vino_server_setup_framebuffer (server);
 
-  client = g_new0 (VinoServerClientInfo, 1);
+  client = g_new0 (VinoClient, 1);
 
   rfb_client->clientData = client;
 
@@ -280,11 +380,6 @@ vino_server_handle_new_client (rfbClientPtr rfb_client)
 
   client->io_channel = g_io_channel_unix_new (rfb_client->sock);
 
-  client->io_watch = g_io_add_watch (client->io_channel,
-				     G_IO_IN|G_IO_PRI,
-				     (GIOFunc) vino_server_client_data_pending,
-				     rfb_client);
-
   server->priv->clients = g_slist_prepend (server->priv->clients, client);
 
   vino_server_set_client_on_hold (server, client, server->priv->on_hold);
@@ -297,13 +392,16 @@ vino_server_handle_prompt_response (VinoServer         *server,
 				    rfbClientPtr        rfb_client,
 				    VinoPromptResponse  response)
 {
+  VinoClient *client = (VinoClient *) rfb_client->clientData;
+
   g_return_if_fail (VINO_IS_SERVER (server));
   g_return_if_fail (rfb_client != NULL);
 
   switch (response)
     {
     case VINO_RESPONSE_ACCEPT:
-      vino_server_set_client_on_hold (server, rfb_client->clientData, FALSE);
+      vino_server_set_client_on_hold (server, client, FALSE);
+      vino_server_client_accepted (server, client);
       break;
     case VINO_RESPONSE_REJECT:
       rfbCloseClient (rfb_client);
@@ -318,13 +416,16 @@ vino_server_handle_prompt_response (VinoServer         *server,
 static enum rfbNewClientAction
 vino_server_handle_authenticated_client (rfbClientPtr rfb_client)
 {
-  VinoServer           *server = VINO_SERVER (rfb_client->screen->screenData);
-  VinoServerClientInfo *client = (VinoServerClientInfo *) rfb_client->clientData;
+  VinoServer *server = VINO_SERVER (rfb_client->screen->screenData);
+  VinoClient *client = (VinoClient *) rfb_client->clientData;
 
   g_return_val_if_fail (VINO_IS_SERVER (server), RFB_CLIENT_REFUSE);
 
   if (!server->priv->prompt_enabled)
-    return RFB_CLIENT_ACCEPT;
+    {
+      vino_server_client_accepted (server, client);
+      return RFB_CLIENT_ACCEPT;
+    }
 
   vino_prompt_add_client (server->priv->prompt, rfb_client);
 
@@ -392,7 +493,7 @@ vino_server_handle_clipboard_event (char         *str,
 }
 
 static gboolean
-vino_server_auth_client_deferred (VinoServerClientInfo *client)
+vino_server_auth_client_deferred (VinoClient *client)
 {
   VinoServer              *server = VINO_SERVER (client->rfb_client->screen->screenData);
   enum rfbNewClientAction  result;
@@ -416,10 +517,10 @@ vino_server_auth_client_deferred (VinoServerClientInfo *client)
 }
 
 static void
-vino_server_defer_client_auth (VinoServer           *server,
-			       VinoServerClientInfo *client,
-			       const char           *response,
-			       int                   length)
+vino_server_defer_client_auth (VinoServer *server,
+			       VinoClient *client,
+			       const char *response,
+			       int         length)
 {
   client->auth_resp_len = length;
   client->auth_response = g_new (char, length);
@@ -432,11 +533,48 @@ vino_server_defer_client_auth (VinoServer           *server,
 					client);
 }
 
+static char *
+vino_server_get_password_from_keyring (VinoServer *server)
+{
+#ifdef VINO_ENABLE_KEYRING
+  GnomeKeyringNetworkPasswordData *found_item;
+  GnomeKeyringResult               result;
+  GList                           *matches;
+  char                            *password;
+  
+  matches = NULL;
+
+  result = gnome_keyring_find_network_password_sync (
+                NULL,           /* user     */
+		NULL,           /* domain   */
+		"vino.local",   /* server   */
+		NULL,           /* object   */
+		"rfb",          /* protocol */
+		"vnc-password", /* authtype */
+		5900,           /* port     */
+		&matches);
+
+  if (result != GNOME_KEYRING_RESULT_OK || matches == NULL || matches->data == NULL)
+    return NULL;
+
+
+  found_item = (GnomeKeyringNetworkPasswordData *) matches->data;
+
+  password = g_strdup (found_item->password);
+
+  gnome_keyring_network_password_list_free (matches);
+
+  return password;
+#else
+  return NULL;
+#endif
+}
+
 static enum rfbNewClientAction
-vino_server_auth_client (VinoServer           *server,
-			 VinoServerClientInfo *client,
-			 const char           *response,
-			 int                   length)
+vino_server_auth_client (VinoServer *server,
+			 VinoClient *client,
+			 const char *response,
+			 int         length)
 {
   rfbClientPtr  rfb_client;
   char         *password;
@@ -444,17 +582,27 @@ vino_server_auth_client (VinoServer           *server,
   if (!(server->priv->auth_methods & VINO_AUTH_VNC))
     goto auth_failed;
 
-  if (!server->priv->vnc_password)
-    goto auth_failed;
-
-  if (!(password = vino_base64_unencode (server->priv->vnc_password)))
+  if (!(password = vino_server_get_password_from_keyring (server)))
     {
-      g_warning ("Failed to base64 unencode VNC password\n");
-      goto auth_failed;
+      guchar *blob;
+      gsize   blob_len;
+
+      if (!server->priv->vnc_password)
+        goto auth_failed;
+
+      blob_len = 0;
+      blob = g_base64_decode (server->priv->vnc_password, &blob_len);
+
+      password = g_strndup ((char *) blob, blob_len);
+
+      g_free (blob);
     }
 
   rfb_client = client->rfb_client;
 
+  if (!password)
+    goto auth_failed;
+
   vncEncryptBytes (client->rfb_client->authChallenge, password);
 
   memset (password, 0, strlen (password));
@@ -486,8 +634,8 @@ vino_server_check_vnc_password (rfbClientPtr  rfb_client,
 				const char   *response,
 				int           length)
 {
-  VinoServer           *server = VINO_SERVER (rfb_client->screen->screenData);
-  VinoServerClientInfo *client = (VinoServerClientInfo *) rfb_client->clientData;
+  VinoServer *server = VINO_SERVER (rfb_client->screen->screenData);
+  VinoClient *client = (VinoClient *) rfb_client->clientData;
 
   g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
 
@@ -601,7 +749,10 @@ vino_server_setup_framebuffer (VinoServer *server)
 			    G_CALLBACK (vino_server_handle_damage_notify),
 			    server);
 
-  server->priv->rfb_screen->frameBuffer = vino_fb_get_pixels (server->priv->fb);
+  rfbNewFramebuffer (server->priv->rfb_screen,
+                     vino_fb_get_pixels (server->priv->fb),
+                     gdk_screen_get_width (server->priv->screen),
+                     gdk_screen_get_height (server->priv->screen));
   
   vino_server_init_pixel_format (server, server->priv->rfb_screen);
 
@@ -628,7 +779,8 @@ static void
 vino_server_init_from_screen (VinoServer *server,
 			      GdkScreen  *screen)
 {
-  rfbScreenInfoPtr rfb_screen;
+  rfbScreenInfoPtr  rfb_screen;
+  char             *name;
 
   g_return_if_fail (server->priv->screen == NULL);
   g_return_if_fail (screen != NULL);
@@ -653,6 +805,15 @@ vino_server_init_from_screen (VinoServer *server,
 		  gdk_screen_get_height (screen),
 		  -1, -1, -1);
 
+  name = g_strjoin ("@",
+                    g_get_user_name (),
+                    g_get_host_name (),
+                    NULL);
+
+  rfbSetDesktopName(rfb_screen, name);
+
+  g_free (name);
+
   /* libvncserver NOTE:
    *   DeferUpdateTime is the number of milliseconds to wait
    *   before actually responding to a frame buffer update
@@ -661,9 +822,17 @@ vino_server_init_from_screen (VinoServer *server,
    *   5900-6000
    */
   rfb_screen->rfbDeferUpdateTime = 0;
+  rfb_screen->localOnly          = server->priv->local_only;
   rfb_screen->autoPort           = TRUE;
+  rfb_screen->rfbPort            = VINO_SERVER_DEFAULT_PORT;
   rfb_screen->rfbAlwaysShared    = TRUE;
 
+  if (server->priv->use_alternative_port)
+    {
+      rfb_screen->autoPort = FALSE;
+      rfb_screen->rfbPort  = server->priv->alternative_port;
+    }
+
   rfbInitServer (rfb_screen);
   
   /* libvncserver NOTE:
@@ -684,6 +853,8 @@ vino_server_init_from_screen (VinoServer *server,
 
   rfb_screen->passwordCheck = vino_server_check_vnc_password;
 
+  vino_server_update_security_types (server);
+
   dprintf (RFB, "Creating watch for listening socket %d - port %d\n",
 	   rfb_screen->rfbListenSock, rfb_screen->rfbPort);
 
@@ -699,6 +870,8 @@ vino_server_init_from_screen (VinoServer *server,
 #endif
 
   vino_mdns_add_service ("_rfb._tcp", rfb_screen->rfbPort);
+
+  server->priv->listener = vino_dbus_listener_new (server);
 }
 
 static void
@@ -748,6 +921,10 @@ vino_server_finalize (GObject *object)
   if (server->priv->fb)
     g_object_unref (server->priv->fb);
   server->priv->fb = NULL;
+
+  if (server->priv->listener)
+    g_object_unref (server->priv->listener);
+  server->priv->listener = NULL;
   
   g_free (server->priv);
   server->priv = NULL;
@@ -778,6 +955,9 @@ vino_server_set_property (GObject      *object,
     case PROP_VIEW_ONLY:
       vino_server_set_view_only (server, g_value_get_boolean (value));
       break;
+    case PROP_LOCAL_ONLY:
+      vino_server_set_local_only (server, g_value_get_boolean (value));
+      break;
     case PROP_REQUIRE_ENCRYPTION:
       vino_server_set_require_encryption (server, g_value_get_boolean (value));
       break;
@@ -787,6 +967,15 @@ vino_server_set_property (GObject      *object,
     case PROP_VNC_PASSWORD:
       vino_server_set_vnc_password (server, g_value_get_string (value));
       break;
+    case PROP_USE_ALTERNATIVE_PORT:
+      vino_server_set_use_alternative_port (server, g_value_get_boolean (value));
+      break;
+    case PROP_ALTERNATIVE_PORT:
+      vino_server_set_alternative_port (server, g_value_get_int (value));
+      break;
+    case PROP_LOCK_SCREEN:
+      vino_server_set_lock_screen (server, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -815,8 +1004,11 @@ vino_server_get_property (GObject    *object,
     case PROP_VIEW_ONLY:
       g_value_set_boolean (value, server->priv->view_only);
       break;
+    case PROP_LOCAL_ONLY:
+      g_value_set_boolean (value, server->priv->local_only);
+      break;
     case PROP_REQUIRE_ENCRYPTION:
-      g_value_set_boolean (value, server->priv->view_only);
+      g_value_set_boolean (value, server->priv->require_encryption);
       break;
     case PROP_AUTH_METHODS:
       g_value_set_flags (value, server->priv->auth_methods);
@@ -824,6 +1016,18 @@ vino_server_get_property (GObject    *object,
     case PROP_VNC_PASSWORD:
       g_value_set_string (value, server->priv->vnc_password);
       break;
+    case PROP_USE_ALTERNATIVE_PORT:
+      g_value_set_boolean (value, server->priv->use_alternative_port);
+      break;
+    case PROP_ALTERNATIVE_PORT:
+      g_value_set_int (value, server->priv->alternative_port);
+      break;
+    case PROP_PORT:
+      g_value_set_int (value, server->priv->rfb_screen->rfbPort);
+      break;
+    case PROP_LOCK_SCREEN:
+      g_value_set_boolean (value, server->priv->lock_screen);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -850,59 +1054,152 @@ vino_server_class_init (VinoServerClass *klass)
   g_object_class_install_property (gobject_class,
 				   PROP_SCREEN,
 				   g_param_spec_object ("screen",
-							_("Screen"),
-							_("The screen for which to create a VNC server"),
+							"Screen",
+							"The screen for which to create a VNC server",
 							GDK_TYPE_SCREEN,
-							G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  
+							G_PARAM_READWRITE      |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_NAME    |
+                                                        G_PARAM_STATIC_NICK    |
+                                                        G_PARAM_STATIC_BLURB));
+
   g_object_class_install_property (gobject_class,
 				   PROP_ON_HOLD,
 				   g_param_spec_boolean ("on-hold",
-							 _("On Hold"),
-							 _("Place all clients on hold"),
+							 "On Hold",
+							 "Place all clients on hold",
 							 TRUE,
-							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (gobject_class,
 				   PROP_PROMPT_ENABLED,
 				   g_param_spec_boolean ("prompt-enabled",
-							 _("Prompt enabled"),
-							 _("Prompt the user about connection attempts"),
+							 "Prompt enabled",
+							 "Prompt the user about connection attempts",
 							 TRUE,
-							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (gobject_class,
 				   PROP_VIEW_ONLY,
 				   g_param_spec_boolean ("view-only",
-							 _("View Only"),
-							 _("Disallow keyboard/pointer input from clients"),
+							 "View Only",
+							 "Disallow keyboard/pointer input from clients",
 							 FALSE,
-							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_LOCAL_ONLY,
+				   g_param_spec_boolean ("local-only",
+							 "Local Only",
+							 "Only allow local connections",
+							 FALSE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
 
   g_object_class_install_property (gobject_class,
 				   PROP_REQUIRE_ENCRYPTION,
 				   g_param_spec_boolean ("require-encryption",
-							 _("Require Encryption"),
-							 _("Require clients to use encryption"),
+							 "Require Encryption",
+							 "Require clients to use encryption",
 							 TRUE,
-							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
 
   g_object_class_install_property (gobject_class,
 				   PROP_AUTH_METHODS,
 				   g_param_spec_flags ("auth-methods",
-						       _("Authentication methods"),
-						       _("The authentication methods this server should allow"),
+						       "Authentication methods",
+						       "The authentication methods this server should allow",
 						       VINO_TYPE_AUTH_METHOD,
 						       VINO_AUTH_NONE,
-						       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                       G_PARAM_READWRITE   |
+                                                       G_PARAM_CONSTRUCT   |
+                                                       G_PARAM_STATIC_NAME |
+                                                       G_PARAM_STATIC_NICK |
+                                                       G_PARAM_STATIC_BLURB));
   
   g_object_class_install_property (gobject_class,
 				   PROP_VNC_PASSWORD,
 				   g_param_spec_string ("vnc-password",
-							_("VNC Password"),
-							_("The password (base64 encoded) used to authenticate types using the VncAuth method"),
+							"VNC Password",
+							"The password (base64 encoded) used to authenticate types using the VncAuth method",
 							NULL,
-							G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                        G_PARAM_READWRITE   |
+                                                        G_PARAM_CONSTRUCT   |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_USE_ALTERNATIVE_PORT,
+				   g_param_spec_boolean ("use-alternative-port",
+							 "Use an alternative port",
+							 "Listen on the port specified by the 'alternative-port' property",
+							 FALSE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_ALTERNATIVE_PORT,
+				   g_param_spec_int ("alternative-port",
+						     "Alternative port number",
+						     "Listen on the specified port number if the 'use-alternative-port' property is TRUE",
+						     VINO_SERVER_MIN_PORT,
+						     VINO_SERVER_MAX_PORT,
+						     VINO_SERVER_DEFAULT_PORT,
+                                                     G_PARAM_READWRITE   |
+                                                     G_PARAM_CONSTRUCT   |
+                                                     G_PARAM_STATIC_NAME |
+                                                     G_PARAM_STATIC_NICK |
+                                                     G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_PORT,
+				   g_param_spec_int ("port",
+						     "Server port number",
+						     "The port used by this server",
+						     VINO_SERVER_MIN_PORT,
+						     VINO_SERVER_MAX_PORT,
+						     VINO_SERVER_DEFAULT_PORT,
+						     G_PARAM_READABLE    |
+                                                     G_PARAM_STATIC_NAME |
+                                                     G_PARAM_STATIC_NICK |
+                                                     G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (gobject_class,
+				   PROP_LOCK_SCREEN,
+				   g_param_spec_boolean ("lock-screen",
+							 "Locks screen on disconnect",
+							 "After last user disconnects, screen will be locked",
+							 FALSE,
+                                                         G_PARAM_READWRITE   |
+                                                         G_PARAM_CONSTRUCT   |
+                                                         G_PARAM_STATIC_NAME |
+                                                         G_PARAM_STATIC_NICK |
+                                                         G_PARAM_STATIC_BLURB));
+
 }
 
 GType
@@ -978,6 +1275,94 @@ vino_server_set_view_only (VinoServer *server,
     }
 }
 
+gboolean
+vino_server_get_local_only (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->local_only;
+}
+
+void
+vino_server_set_local_only (VinoServer *server,
+                            gboolean    local_only)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  local_only = local_only != FALSE;
+
+  if (server->priv->local_only != local_only)
+    {
+      server->priv->local_only = local_only;
+
+      if (server->priv->rfb_screen != NULL)
+        rfbSetLocalOnly (server->priv->rfb_screen,
+                         server->priv->local_only);
+
+      g_object_notify (G_OBJECT (server), "local-only");
+    }
+}
+
+gboolean
+vino_server_get_use_alternative_port (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->use_alternative_port;
+}
+
+void
+vino_server_set_use_alternative_port (VinoServer *server,
+                                      gboolean    use_alternative_port)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  use_alternative_port = use_alternative_port != FALSE;
+
+  if (server->priv->use_alternative_port != use_alternative_port)
+    {
+      server->priv->use_alternative_port = use_alternative_port;
+
+      if (server->priv->rfb_screen)
+        {
+          if (server->priv->use_alternative_port)
+            rfbSetPort (server->priv->rfb_screen,
+                        server->priv->alternative_port);
+
+          rfbSetAutoPort (server->priv->rfb_screen,
+                          !server->priv->use_alternative_port);
+        }
+
+      g_object_notify (G_OBJECT (server), "use-alternative-port");
+    }
+}
+
+gint
+vino_server_get_alternative_port (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->alternative_port;
+}
+
+void
+vino_server_set_alternative_port (VinoServer *server,
+                                  int         alternative_port)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  if (server->priv->alternative_port != alternative_port)
+    {
+      server->priv->alternative_port = alternative_port;
+
+      if (server->priv->rfb_screen &&
+          server->priv->use_alternative_port)
+        rfbSetPort (server->priv->rfb_screen, server->priv->alternative_port);
+
+      g_object_notify (G_OBJECT (server), "alternative-port");
+    }
+}
+
 gboolean
 vino_server_get_on_hold (VinoServer *server)
 {
@@ -1002,7 +1387,7 @@ vino_server_set_on_hold (VinoServer *server,
 
       for (l = server->priv->clients; l; l = l->next)
 	{
-	  VinoServerClientInfo *client = l->data;
+	  VinoClient *client = l->data;
 
 	  /* If a client is on hold before we have initialized,
 	   * we want to leave it on hold until the deferred
@@ -1047,6 +1432,9 @@ vino_server_set_prompt_enabled (VinoServer *server,
 static void
 vino_server_update_security_types (VinoServer *server)
 {
+  if (!server->priv->rfb_screen)
+    return;
+
   rfbClearSecurityTypes (server->priv->rfb_screen);
   rfbClearAuthTypes (server->priv->rfb_screen);
 
@@ -1144,3 +1532,48 @@ vino_server_get_vnc_password (VinoServer *server)
 
   return server->priv->vnc_password;
 }
+
+G_CONST_RETURN char *
+vino_client_get_hostname (VinoClient *client)
+{
+  return client->rfb_client->host;
+}
+
+void
+vino_client_disconnect (VinoClient *client)
+{
+  rfbCloseClient (client->rfb_client);
+  rfbClientConnectionGone (client->rfb_client);
+}
+
+int
+vino_server_get_port (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), 0);
+
+  return server->priv->rfb_screen->rfbPort;
+}
+
+gboolean
+vino_server_get_lock_screen (VinoServer *server)
+{
+  g_return_val_if_fail (VINO_IS_SERVER (server), FALSE);
+
+  return server->priv->lock_screen;
+}
+
+void
+vino_server_set_lock_screen (VinoServer *server,
+			    gboolean    lock_screen)
+{
+  g_return_if_fail (VINO_IS_SERVER (server));
+
+  lock_screen = lock_screen != FALSE;
+
+  if (server->priv->lock_screen != lock_screen)
+    {
+      server->priv->lock_screen = lock_screen;
+
+      g_object_notify (G_OBJECT (server), "lock-screen");
+    }
+}
